package net.sf.basedb.examples.extensions.log;

import java.util.Arrays;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.LogControl;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.NameableData;
import net.sf.basedb.core.log.ChangeType;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.TransactionDetails;
import net.sf.basedb.core.log.db.DbLogManager;

/**
	This log manager write log information to the standard output. It is based on the
	database log manager but overrides the 
	{@link #logChangeDetails(ChangeHistoryDetailData, EntityDetails)} method and
	write the changes to stdout instead of saving it to the database.
	
	@author nicklas
	@since 1.5
*/
public class StdoutLogManager 
	extends DbLogManager 
{
	
	public StdoutLogManager(LogControl logControl)
	{
		super(logControl);
	}

	/**
		Log the given details to stdout instead of saving it to the database.
	*/
	@Override
	public void logChangeDetails(ChangeHistoryDetailData changeDetails, EntityDetails entityDetails)
	{
		LogControl logControl = getLogControl();
		TransactionDetails tx = logControl.getTransactionDetails();
		
		StringBuilder log = new StringBuilder();
		
		// Log the user that made the change
		log.append("User: ").append(tx.getUserName()).append("(").append(tx.getUserId()).append(")");
		
		// If a project was active, save that info to
		if (tx.getProjectId() != 0)
		{
			log.append("; Project: ").append(tx.getProjectName()).append("(").append(tx.getProjectId()).append(")");
		}
		
		// If the change was made by a job and plug-in we log that as well
		if (tx.getPluginId() != 0)
		{
			log.append("; Plugin: ").append(tx.getPluginName()).append("(").append(tx.getPluginId()).append(")");
			log.append("; Job: ").append(tx.getJobName()).append("(").append(tx.getJobId()).append(")");
		}
		
		// The ChangeType is CREATE, UPDATE or DELETE
		log.append("; ").append(ChangeType.fromValue(changeDetails.getChangeType()).name());
		
		// Log information about the item that was created, changed or deleted
		log.append("; ").append(Item.fromValue(changeDetails.getItemType()).toString()).append(": ").append(changeDetails.getItemId());
		if (changeDetails.getChangeInfo() != null)
		{
			log.append("; ").append(changeDetails.getChangeInfo());
		}
		
		// Finally, if the item has a name, we log the old (if it has been modified) and new name
		if (entityDetails.getEntity() instanceof NameableData)
		{
			int nameIndex = Arrays.asList(entityDetails.getProperties()).indexOf("name");
			if (nameIndex >= 0)
			{
				if (entityDetails.getDirty(nameIndex) == 1)
				{
					log.append("; Name: ").append(entityDetails.getPreviousState()[nameIndex]);
					log.append(" --> ").append(entityDetails.getState()[nameIndex]);
				}
				else
				{
					log.append("; Name: ").append(entityDetails.getState()[nameIndex]);
				}
			}
		}
		
		System.out.println(log);
	}
	
}
